*
*   File name:   lpeflpt1.asm
*
*   Description: filtro passa-basso ellittico realizzato in forma
*	parallela. Versione NON REAL-TIME con coefficienti NON SCALATI.
*	La realizzazione soffre di problemi di overflow ad alcune frequenze. Ad
*	esempio un segnale del tipo sin(1000Hz)+sin(3500Hz) viene correttamente
*	filtrato, mentre uno del tipo sin(500Hz)+sin(3500Hz) presenta delle spurie
*	a 1500Hz e 2500Hz.
*
*   Specifiche di progetto:
*
*		fp = 2 kHz	DBR < .5 dB
*		fs = 3 kHz	Att < 40 dB
*
*   Caratteristiche del filtro:
*
*		fp = 2kHz	DBR = .5 dB
*		fs = 3kHz	Att = 56 dB
*
*   Last mofified: 16/06/98
*

	.title   "Low pass elliptic filter: DBR = .5 dB, not scaled, not real-time"
	.mmregs
	.width   80
	.length  60

	.setsect ".text",   1800h, 0
	.setsect ".data",   0200h, 1
	.setsect ".bss",    0210h, 1
	.setsect "vectors", 0180h, 0


	.sect "vectors"			; interrupt vector table stored at
AC01_enable .set 0
	.copy "lpefvecs.asm"		; location 0x0180

	.sect ".data"

*
*	|------------------------------------->|
*	|                                      |
*	|                                      |
*   --->1----------->4----------->4--- b10 --->2--- G --->3
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a11 ---5--- b11 --->2
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a12 ---6            |
*	|                                      |
*	|                                      |
*	|----------->7----------->7--- b20 --->2
*	             |            |            |
*	             |            |            |
*	             |<-- -a21 ---8--- b21 --->|
*	             |            |
*	             |            |
*	             |<-- -a22 ---9
*

COEFF
	.word	-8417	; b11/2
	.word	24022	; a12
	.word	1735	; a11
	.word	-8526	; b10
	.word	23876	; b21/2
	.word	7519	; a22
	.word	-17633	; a21
	.word	-18729	; b20
	.word	16740	; G

STATUS	.word	0			; W11
	.word	0			; W12
	.word	0			; W21
	.word	0			; W22

inbuf	.set 2000h
length	.set 020h
outbuf	.set 2400h


	.sect ".text"
start:

	intm = 1		; disable all interrupts
	DP = #0
	nop
	pmst = #01a0h		; Interrupt pointer maps vectors to address 180h
	sp = #0ffah		; stack pointer located in Communications Kernal
	imr = #240h		; unmask TDM RINT and HPIINT(host port interface)
	intm = 0		; enable all interrupts
	FRCT = #1


	.asg AR1, in
	.asg AR2, out
	.asg AR3, coeff
	.asg AR4, status
	in = #inbuf
	out= #outbuf
	BK = #length
LOOP:
	coeff  = #COEFF
	status = #STATUS

inpp
	B = *in<<16			; y(n)=x(n)
	@BRC = #1			; due celle di 2o grado
	blockrepeat(end_cell)
begin_cell
        B += *coeff * *status		; y(n)+=(bi1/2)*Wi1
        B += *coeff+ * *status+		; y(n)+=(bi1/2)*Wi1
	A = *in<<16			; y'=x(n)
	A -= *coeff+ * *status-		; y'-=ai2*Wi2
	A = rnd(A - *status * *coeff+)	; y'-=ai1*Wi1 (T=Wi1)
	*status+ = hi(A)		; Wi1=y'
	*status+ = T			; Wi2=T

	B += *coeff+ * hi(A)		; y(n)+=bi0*y'
end_cell

	A = B				; Oss.: errore di troncamento. Il
	B = *coeff * hi(A)		; processore non supporta
					; l'istruzione `A = rnd(B)'

	*out+% = hi(B)
outpp
	goto LOOP

	.end
